@@ -2,29 +2,28 @@ library(dendroTools)
 library(testthat)
 library(dplyr)
 
-data(daily_temperatures_example)
+data(LJ_daily_temperatures)
 data(example_proxies_1)
 MVA_parameter <- select(example_proxies_1, MVA)
-oxygen_isotope <- select(example_proxies_1, O)
-carbon_isotope <- select(example_proxies_1, C)
-
+TRW_parameter <- select(example_proxies_1, TRW)
 
 # If test is repeated, equal result should be obtained
-test1 <- daily_response(response = carbon_isotope,
-                        env_data = daily_temperatures_example, method = "lm",
-                        lower = 240, upper = 250)
+test1 <- daily_response(response = MVA_parameter,
+                        env_data = LJ_daily_temperatures, method = "lm",
+                        lower = 80, upper = 120, row_names_subset = TRUE)
 
-test2 <- daily_response(response = carbon_isotope,
-                        env_data = daily_temperatures_example, method = "lm",
-                        lower = 240, upper = 250)
+test2 <- daily_response(response = MVA_parameter,
+                        env_data = LJ_daily_temperatures, method = "lm",
+                        lower = 80, upper = 120, row_names_subset = TRUE)
 
 expect_equal(test1, test2)
 
 
 # daily_response function should return a list with matrix and two characters
 test3 <- daily_response(response = example_proxies_1,
-  env_data = daily_temperatures_example, method = "lm",
-  measure = "adj.r.squared", lower = 250, upper = 253, previous_year = TRUE)
+  env_data = LJ_daily_temperatures, method = "lm",
+  measure = "adj.r.squared", lower = 250, upper = 253, previous_year = TRUE,
+  row_names_subset = TRUE)
 expect_is(test3, "list")
 expect_is(test3[[1]], "matrix")
 expect_is(test3[[2]], "character")
@@ -32,27 +31,29 @@ expect_is(test3[[2]], "character")
 
 
 # stop functions were included to prevent wrong results
-expect_error(daily_response(response = carbon_isotope,
-                            env_data = daily_temperatures_example,
+expect_error(daily_response(response = TRW_parameter,
+                            env_data = LJ_daily_temperatures,
                             lower = 200, upper = 270, fixed_width = -368))
 
 expect_error(daily_response(response = example_proxies_1,
-  env_data = daily_temperatures_example, method = "cor", lower = 250,
+  env_data = LJ_daily_temperatures, method = "cor", lower = 250,
   upper = 270, previous_year = FALSE))
 
 expect_error(daily_response(response = example_proxies_1,
-  env_data = daily_temperatures_example, method = "cor", lower = 280,
+  env_data = LJ_daily_temperatures, method = "cor", lower = 280,
   upper = 270, previous_year = FALSE))
 
 
 # r.squared is squared correlation. Results should be equal
 test4 <- daily_response(response = MVA_parameter,
-                        env_data = daily_temperatures_example, method = "cor",
-                        lower = 150, upper = 170, previous_year = FALSE)
+                        env_data = LJ_daily_temperatures, method = "cor",
+                        lower = 150, upper = 170, previous_year = FALSE,
+                        row_names_subset = TRUE)
 
 test5 <- daily_response(response = MVA_parameter,
-                        env_data = daily_temperatures_example, method = "lm",
-                        lower = 150, upper = 170, previous_year = FALSE)
+                        env_data = LJ_daily_temperatures, method = "lm",
+                        lower = 150, upper = 170, previous_year = FALSE,
+                        row_names_subset = TRUE)
 expect_equal(max(test4[[1]], na.rm = TRUE) ^ 2, max(test5[[1]], na.rm = TRUE))
 
 
@@ -74,31 +75,32 @@ example_proxies_1_temporal <- example_proxies_1
 row.names(example_proxies_1_temporal)[1] <- "1520" # random year is assigned
 # to row.name of the firest row
 expect_warning(daily_response(response = example_proxies_1_temporal,
-                            env_data = daily_temperatures_example,
+                            env_data = LJ_daily_temperatures,
                             method = "lm", lower = 260, upper = 270,
                             previous_year = FALSE))
 
 
 # The order of data is unimportant, if row_names_subset is set to TRUE and
 # row.names are years. In this case, data is merged based on matching
-# row .ames.
+# row names.
 # will be ordered data
 data(example_proxies_1)
-carbon_isotope <- dplyr::select(example_proxies_1, C)
+MVA_parameter <- dplyr::select(example_proxies_1, MVA)
 # Test 8: Usual way of analysing data
-test8 <- daily_response(response = carbon_isotope,
-                           env_data = daily_temperatures_example, method = "lm",
-                        measure = "r.squared", lower_limit = 30,
-                        upper_limit = 40)
-# Test 9: carbon_isotope data.frame is ordered by values of C
-carbon_isotope_ordered <- carbon_isotope[order(carbon_isotope$C), ,
+test8 <- daily_response(response = MVA_parameter,
+                           env_data = LJ_daily_temperatures, method = "lm",
+                        measure = "r.squared", lower_limit = 50,
+                        upper_limit = 80, row_names_subset = TRUE)
+
+# Test 9: MVA_parameter data.frame is ordered by values of MVA
+MVA_parameter_ordered <- MVA_parameter[order(MVA_parameter$MVA), ,
                                          drop = FALSE]
-test9 <- daily_response(response = carbon_isotope_ordered,
-                           env_data = daily_temperatures_example, method = "lm",
-                        measure = "r.squared", lower_limit = 30,
-                        upper_limit = 40, row_names_subset = TRUE)
+test9 <- daily_response(response = MVA_parameter_ordered,
+                           env_data = LJ_daily_temperatures, method = "lm",
+                        measure = "r.squared", lower_limit = 50,
+                        upper_limit = 80, row_names_subset = TRUE)
 expect_equal(test8[[1]], test9[[1]])
 
 
 # There should be 4 elements in a list returned by daily_response()
-expect_equal(length(test8), 4)
+expect_equal(length(test8), 5)
